@@ -3,6 +3,8 @@
 from argparse import ArgumentParser
 import sys
 import xemc3
+import os
+import xarray as xr
 
 long_names = {
     "f_n": "Particle flux",
@@ -28,8 +30,11 @@ def get_key(name):
 def plot(cwd, args):
     while cwd[-1] == "/":
         cwd = cwd[:-1]
-    cwd += "/"
-    plates = xemc3.load.plates(cwd)
+    if os.path.isdir(cwd):
+        cwd += "/"
+        plates = xemc3.load.plates(cwd)
+    else:
+        plates = xr.open_dataset(cwd)
     key = args.key
     if key not in plates:
         key = get_key(key)
@@ -111,13 +116,13 @@ def parser() -> ArgumentParser:
         "--key",
         default="Energy flux",
         type=get_name,
-        choices=[
-            "Particle flux",
-            "Energy flux",
-            "Averge density",
-            "Average electron temperature",
-            "Average ion temperature",
-        ],
+        # choices=[
+        #     "Particle flux",
+        #     "Energy flux",
+        #     "Averge density",
+        #     "Average electron temperature",
+        #     "Average ion temperature",
+        # ],
         help="Data to plot",
     )
     parser.add_argument("path", nargs=1, help="Path of data")
