@@ -1,6 +1,6 @@
 """Routines for reading EMC3 files."""
 
-__all__ = ["plates", "mappings", "all"]
+__all__ = ["plates", "mappings", "all", "mapped_raw", "file", "var", "plates_geom"]
 
 import sys as _sys
 import types as _types
@@ -13,6 +13,7 @@ from ..core.load import (
     load_all as all,
     read_var as var,
     read_plate_nice as plates_geom,
+    load_any as any,
     # Removed in 0.0.5
     #    read_mappings as mappings,
     #    read_mapped_nice as mapped,
@@ -25,15 +26,15 @@ from ..core.load import (
 # write_all_fortran as _write_all,
 
 
-class _all(_types.ModuleType):
+class _any(_types.ModuleType):
     def __init__(self):
         super().__init__(__name__)
         self.__dict__.update(_sys.modules[__name__].__dict__)
 
     def __call__(self, path, ignore_missing=None):
-        return all(path, ignore_missing)
+        return any(path, ignore_missing)
 
 
-_all.__call__.__doc__ = all.__doc__
+_any.__call__.__doc__ = any.__doc__
 
-_sys.modules[__name__] = _all()
+_sys.modules[__name__] = _any()
