@@ -3,10 +3,11 @@ import os
 import pytest
 from xarray.testing import assert_identical
 import xarray as xr
+import numpy as np
 
 
 def get_data():
-    basedir = "xemc3/test/testdata/"
+    basedir = "./example-data/"
     if not os.path.isdir(basedir):
         pytest.skip("create {basedir} to enable testing on real data")
         return
@@ -16,7 +17,7 @@ def get_data():
         )
     else:
         os.system(f"cd {basedir}; git fetch origin main --depth 1")
-        os.system(f"cd {basedir}; git checkout origin next")
+        os.system(f"cd {basedir}; git checkout origin main")
     return basedir + "emc3_example"
 
 
@@ -25,6 +26,11 @@ def test_load_all():
     bd = get_data()
     result = xemc3.load.all(bd)
     expected = xr.open_dataset(bd + ".nc")
+    # Remove new attributes, so we don't have to regenerate the data that often
+    for k in list(result) + list(result.coords):
+        for a in [a for a in result[k].attrs]:
+            if a not in expected[k].attrs:
+                del result[k].attrs[a]
     assert_identical(result, expected)
 
 
@@ -39,8 +45,15 @@ def test_load_plates():
 def test_iter_plates():
     bd = get_data()
     ds = xemc3.load.plates(bd)
-    for _ in ds.emc3.iter_plates():
-        pass
+    c = 0
+    for x in ds.emc3.iter_plates():
+        c += 1
+        assert np.all(np.isfinite(x.emc3["f_E"].data))
+    assert c == 22
+    c = 0
+    for _ in ds.emc3.iter_plates(symmetry=True, segments=5):
+        c += 1
+    assert c == 22 * 5 * 2
 
 
 def test_get_element():
